/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: CarMetadataMerge.java 4672 2009-02-20 15:24:30Z sauthieg $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.car.impl.xml.merge;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.jar.Attributes;

import org.ow2.util.archive.api.IArchiveMetadata;
import org.ow2.util.ee.metadata.car.api.ICarClassMetadata;
import org.ow2.util.ee.metadata.car.api.ICarFieldMetadata;
import org.ow2.util.ee.metadata.car.api.ICarMethodMetadata;
import org.ow2.util.ee.metadata.car.impl.CarDeployableMetadata;
import org.ow2.util.ee.metadata.car.impl.xml.struct.CAR;
import org.ow2.util.ee.metadata.common.api.ICommonClassMetadata;
import org.ow2.util.ee.metadata.common.api.ICommonFieldMetadata;
import org.ow2.util.ee.metadata.common.api.ICommonMethodMetadata;
import org.ow2.util.ee.metadata.common.api.interfaces.IEnvironmentRefAccessor;
import org.ow2.util.ee.metadata.common.api.struct.IJaxwsWebServiceRef;
import org.ow2.util.ee.metadata.common.impl.xml.merge.MetadataMerge;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * CarMetadataMerge.
 * @author Gael Lalire
 */
public class CarMetadataMerge extends MetadataMerge {

    /**
     * Logger.
     */
    private static final Log LOGGER = LogFactory.getLog(CarMetadataMerge.class);

    /**
     * The metadata where merge will be done.
     */
    private CarDeployableMetadata carDeployableMetadata;

    /**
     * Constructor.
     * @param carDeployableMetadata the metadata where merge will be done
     */
    public CarMetadataMerge(final CarDeployableMetadata carDeployableMetadata) {
        this.carDeployableMetadata = carDeployableMetadata;
    }

    /**
     * Merge current metadata with car.
     * @param car object representing the xml
     */
    public void merge(final CAR car) {
        applyJndiEnvironmentRefsGroup(car, carDeployableMetadata);

        // only in CAR
        String callbackHandler = car.getCallbackHandler();
        if (callbackHandler != null) {
            ICarClassMetadata carClassMetadata = getClassMetadata(callbackHandler.replace('.', '/'));
            if (carClassMetadata == null) {
                LOGGER.error("The callback handler class '" + callbackHandler + "' defined in xml was not found.");
            } else {
                carClassMetadata.setCallbackHandler(true);
            }
        }
    }

    /**
     * @param className the class name
     * @return class metadata of class name
     */
    @Override
    public ICarClassMetadata getClassMetadata(final String className) {
        return carDeployableMetadata.getCarClassMetadata(className);
    }

    /**
     * Provide the main-class of the client.
     * @param env the car environment
     * @return the main-class of the client archive
     */
    @Override
    public String findDefaultClassForEnvironment(final IEnvironmentRefAccessor env) {
        IArchiveMetadata metadata = carDeployableMetadata.getDeployable().getArchive().getMetadata();
        String mainClassName = metadata.get(Attributes.Name.MAIN_CLASS.toString());
        if (mainClassName == null) {
            // try with the attribute name in lowercase (not spec compliant...)
            mainClassName = metadata.get("main-class");
            if (mainClassName != null) {
                LOGGER.warn("The '" + Attributes.Name.MAIN_CLASS.toString()
                        + "' attribute is not written correctly in the archive '"
                        + carDeployableMetadata.getDeployable().getArchive() + "'");
            }

        }
        if (mainClassName == null) {
            throw new IllegalStateException("No '" + Attributes.Name.MAIN_CLASS.toString()
                    + "' attribute found in the archive '" + carDeployableMetadata.getDeployable().getArchive() + "'");
        }

        return encode(mainClassName);
    }


    /**
     * Needs to search the WebServiceRef in all fields/methods of the CAR.
     * @param carMetadata the car metadata
     * @param name the xml name of the service ref
     * @return a list of matching web service ref
     */
    @Override
    protected List<IJaxwsWebServiceRef> findJaxwsWebServiceRefMetadata(final IEnvironmentRefAccessor carMetadata,
            final String name) {

        List<IJaxwsWebServiceRef> elements = new ArrayList<IJaxwsWebServiceRef>();

        Collection<ICarClassMetadata> clientClassesMetadata = carDeployableMetadata.getCarClassMetadataCollection();
        if (clientClassesMetadata != null) {
            for (ICarClassMetadata classMetadata : clientClassesMetadata) {
                Collection<? extends ICarFieldMetadata> fieldMetadatas = classMetadata.getStandardFieldMetadataCollection();
                if (fieldMetadatas != null) {
                    for (ICarFieldMetadata fieldMetadata : fieldMetadatas) {
                        addJaxwsWebServiceRefIfFound(fieldMetadata, name, elements);
                    }
                }
                Collection<? extends ICarMethodMetadata> methodMetadatas = classMetadata.getStandardMethodMetadataCollection();
                if (methodMetadatas != null) {
                    for (ICarMethodMetadata methodMetadata : methodMetadatas) {
                        addJaxwsWebServiceRefIfFound(methodMetadata, name, elements);
                    }
                }
            }
        }
        return elements;
    }

    /**
     * @return A collection of all ClassMetadata of the merged object.
     */
    protected Collection<? extends ICommonClassMetadata> getAllClassMetadatas() {
        return carDeployableMetadata.getCarClassMetadataCollection();
    }

    /**
     * @return A collection of all MethodMetadata of the merged object.
     */
    protected Collection<? extends ICommonMethodMetadata> getAllMethodMetadatas() {
        // Init the returned List
        Collection<ICommonMethodMetadata> methodMetadatas = new ArrayList<ICommonMethodMetadata>();

        Collection<? extends ICommonClassMetadata> classMetadatas = carDeployableMetadata.getCarClassMetadataCollection();
        for (ICommonClassMetadata classMetadata : classMetadatas) {
            methodMetadatas.addAll(classMetadata.getStandardMethodMetadataCollection());
        }
        return methodMetadatas;
    }

    /**
     * @return A collection of all FieldMetadata of the merged object.
     */
    protected Collection<? extends ICommonFieldMetadata> getAllFieldMetadatas() {
        // Init the returned List
        Collection<ICommonFieldMetadata> fieldMetadatas = new ArrayList<ICommonFieldMetadata>();

        Collection<? extends ICommonClassMetadata> classMetadatas = carDeployableMetadata.getCarClassMetadataCollection();
        for (ICommonClassMetadata classMetadata : classMetadatas) {
            fieldMetadatas.addAll(classMetadata.getStandardFieldMetadataCollection());
        }
        return fieldMetadatas;
    }
}
